@@ -1,25 +1,21 @@
 # pylint: disable=invalid-name
-"""
-Current vector control for synchronous motor drives.
+"""Current vector control for synchronous motor drives."""
 
-"""
 from __future__ import annotations
 from collections.abc import Callable
 from dataclasses import dataclass, field
 import numpy as np
 
 from motulator.helpers import abc2complex, Bunch
-from motulator.control.common import SpeedCtrl, PWM
+from motulator.control.common import Ctrl, SpeedCtrl, PWM
 from motulator.control.sm_torque import TorqueCharacteristics
 
 
 # %%
 @dataclass
 class SynchronousMotorVectorCtrlPars:
-    """
-    Vector control parameters for synchronous motors.
+    """Vector control parameters for synchronous motors."""
 
-    """
     # pylint: disable=too-many-instance-attributes
     # Speed reference (in electrical rad/s)
     w_m_ref: Callable[[float], float] = field(
@@ -69,7 +65,7 @@ class SynchronousMotorVectorCtrlPars:
 
 
 # %%
-class SynchronousMotorVectorCtrl:
+class SynchronousMotorVectorCtrl(Ctrl):
     """
     Vector control for a synchronous motor drive.
 
@@ -84,9 +80,8 @@ class SynchronousMotorVectorCtrl:
     """
 
     # pylint: disable=too-many-instance-attributes
-    def __init__(self, pars=SynchronousMotorVectorCtrlPars()):
-
-        self.t = 0
+    def __init__(self, pars):
+        super().__init__()
         self.T_s = pars.T_s
         self.w_m_ref = pars.w_m_ref
         self.p = pars.p
@@ -99,11 +94,10 @@ class SynchronousMotorVectorCtrl:
             self.observer = SensorlessObserver(pars)
         else:
             self.observer = None
-        self.data = Bunch()
 
     def __call__(self, mdl):
         """
-        Main control loop.
+        Run the main control loop.
 
         Parameters
         ----------
@@ -127,9 +121,11 @@ class SynchronousMotorVectorCtrl:
         u_dc = mdl.conv.meas_dc_voltage()  # DC-bus voltage
 
         if not self.sensorless:
-            # Measure the rotor speed and position
+            # Measure the rotor speed
             w_m = self.p*mdl.mech.meas_speed()
-            theta_m = self.p*np.mod(mdl.mech.meas_position(), 2*np.pi)
+            # Limit the electrical rotor position into [-pi, pi)
+            theta_m = np.mod(
+                self.p*mdl.mech.meas_position() + np.pi, 2*np.pi) - np.pi
         else:
             # Get the rotor speed and position estimates
             w_m, theta_m = self.observer.w_m, self.observer.theta_m
@@ -147,10 +143,18 @@ class SynchronousMotorVectorCtrl:
         d_abc_ref, u_s_ref_lim = self.pwm.output(u_s_ref, u_dc, theta_m, w_m)
 
         # Data logging
-        data = Bunch(i_s_ref=i_s_ref, i_s=i_s, u_s=u_s, w_m_ref=w_m_ref,
-                     w_m=w_m, theta_m=theta_m, u_dc=u_dc,
-                     tau_M_ref_lim=tau_M_ref_lim, t=self.t)
-        self._save(data)
+        data = Bunch(
+            i_s=i_s,
+            i_s_ref=i_s_ref,
+            t=self.t,
+            tau_M_ref_lim=tau_M_ref_lim,
+            theta_m=theta_m,
+            u_dc=u_dc,
+            u_s=u_s,
+            w_m=w_m,
+            w_m_ref=w_m_ref,
+        )
+        self.save(data)
 
         # Update states
         if self.sensorless:
@@ -159,22 +163,10 @@ class SynchronousMotorVectorCtrl:
         self.current_ref.update(tau_M_ref_lim, u_s_ref, u_dc)
         self.current_ctrl.update(u_s_ref_lim, w_m)
         self.pwm.update(u_s_ref_lim)
-        self.t += self.T_s
+        self.update_clock(self.T_s)
 
         return self.T_s, d_abc_ref
 
-    def _save(self, data):
-        for key, value in data.items():
-            self.data.setdefault(key, []).extend([value])
-
-    def post_process(self):
-        """
-        Transform the lists to the ndarray format.
-
-        """
-        for key in self.data:
-            self.data[key] = np.asarray(self.data[key])
-
 
 # %%
 class CurrentCtrl:
@@ -337,6 +329,7 @@ class CurrentRef:
             Limited torque reference.
 
         """
+
         def limit_torque(tau_M_ref, w_m, u_dc):
             if np.abs(w_m) > 0:
                 psi_s_max = self.k_u*u_dc/np.sqrt(3)/np.abs(w_m)
@@ -354,15 +347,14 @@ class CurrentRef:
 
         # q-axis current reference
         psi_t = self.psi_f + (self.L_d - self.L_q)*self.i_sd_ref
-        if psi_t != 0:
-            i_sq_ref = tau_M_ref/(1.5*self.p*psi_t)
-        else:
-            i_sq_ref = 0
+        i_sq_ref = tau_M_ref/(1.5*self.p*psi_t) if psi_t != 0 else 0
 
         # Limit the q-axis current reference
         i_sd_mtpa = self.i_sd_mtpa(np.abs(tau_M_ref))
-        i_sq_max = np.min([np.sqrt(self.i_s_max**2 - self.i_sd_ref**2),
-                           np.sqrt(self.i_s_max**2 - i_sd_mtpa**2)])
+        i_sq_max = np.min([
+            np.sqrt(self.i_s_max**2 - self.i_sd_ref**2),
+            np.sqrt(self.i_s_max**2 - i_sd_mtpa**2)
+        ])
         if np.abs(i_sq_ref) > i_sq_max:
             i_sq_ref = np.sign(i_sq_ref)*i_sq_max
 
@@ -461,12 +453,11 @@ class SensorlessObserver:
         # Pole locations are chosen according to (36), with c = w_m**2
         # and w_inf = inf, and the gain corresponding to (30) is used
         k = self.b_p + 2*self.zeta_inf*np.abs(self.w_m)
-        psi_a_sqr = np.abs(psi_a)**2
-        if psi_a_sqr > 0:
+        if np.abs(psi_a) > 0:
             # Correction voltage
-            v = k*psi_a*np.real(psi_a*np.conj(e))/psi_a_sqr
+            v = k*psi_a*np.real(e/psi_a)
             # Error signal (10)
-            eps = np.imag(psi_a*np.conj(e))/psi_a_sqr
+            eps = -np.imag(e/psi_a)
         else:
             v, eps = 0, 0
 
@@ -476,5 +467,5 @@ class SensorlessObserver:
         # Update the states
         self.psi_s += self.T_s*(u_s - self.R_s*i_s - 1j*w_m*self.psi_s + v)
         self.w_m += self.T_s*self.k_i*eps
-        self.theta_m += self.T_s*w_m
-        self.theta_m = np.mod(self.theta_m, 2*np.pi)  # Limit to [0, 2*pi]
+        self.theta_m += self.T_s*w_m  # Next line: limit into [-pi, pi)
+        self.theta_m = np.mod(self.theta_m + np.pi, 2*np.pi) - np.pi
